/*
  rdesktop: A Remote Desktop Protocol client.
  Amiga front-end etc
  Copyright (C) Michal Zukowski 2012-2021
  Copyright (C) Martin Blom 2004-2005

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <proto/wb.h>	
#include <proto/dos.h>
#include <workbench/startup.h>
#include <workbench/workbench.h>
#include <proto/exec.h>

extern struct DiskObject* amiga_icon;

char *parse_wbargs(char **argv)
{
	struct DiskObject* icon;
	BPTR cd;
	int length = 1; // space for one nul character
	int i, t;
	char *wbargs = NULL;
	static struct WBStartup*  wb_msg = NULL;
	
	cd = CurrentDir(0);

	wb_msg = (struct WBStartup*) argv;

	for(i = wb_msg->sm_NumArgs - 1; i >= 0; --i)
	{
		CurrentDir(wb_msg->sm_ArgList[ i ].wa_Lock);

		icon = GetDiskObject(wb_msg->sm_ArgList[ i ].wa_Name);

		if(amiga_icon == NULL)
		{
			amiga_icon = icon;
		}

		if(icon != NULL && icon->do_ToolTypes != NULL)
		{
			for(t = 0;  icon->do_ToolTypes[ t ] != NULL; ++t)
			{
				char* tt = icon->do_ToolTypes[ t ];

				if(isalpha(tt[ 0 ]) && strncmp(tt, "WINDOW", 6) != 0)
				{
					// Add length + space
					length = length + strlen(tt) + 1;
				}
			}

			if(icon != amiga_icon)
			{
				FreeDiskObject(icon);
			}
		}
	}

	if((wbargs = AllocVec(length, MEMF_ANY)))
	{
		wbargs[0] = 0;
		for(i = wb_msg->sm_NumArgs - 1; i >= 0; --i)
		{
			CurrentDir(wb_msg->sm_ArgList[ i ].wa_Lock);

			icon = GetDiskObject(wb_msg->sm_ArgList[ i ].wa_Name);

			if(icon != NULL && icon->do_ToolTypes != NULL)
			{
				for(t = 0;  icon->do_ToolTypes[ t ] != NULL; ++t)
				{
					char* tt = icon->do_ToolTypes[ t ];

					if(isalpha(tt[ 0 ]) && Strnicmp(tt, "WINDOW", 6) != 0)
					{
						int tt_len;
						char* args = wbargs;

						for(tt_len = 0; isalnum(tt[ tt_len ]); ++tt_len);

						while(1)
						{
							if(Strnicmp(args, tt, tt_len) == 0 &&
									! isalnum(tt[ tt_len ]))
							{
								break;
							}
							else
							{
								while(*args != 0)
								{
									++args;

									if(*args == '\n')
									{
										++args;
										break;
									}
								}

								if(*args == 0)
								{
									strcat(args, tt);
									strcat(args, "\n");   // Marker!
									break;
								}
							}
						}
					}
				}

				FreeDiskObject(icon);
			}
		}
	}

	CurrentDir(cd);
	
	return wbargs;
}